# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import time
import pango
import gobject

import UserMenu
import GroupMenu
import TreeViewTooltips
from Parser import UserListDataType
import SmileyRenderer
import FancyAvatarRenderer

import emesenelib.common

import traceback

class UserListModelTree(gtk.TreeStore):
    '''this class is the model for a tree like userlist'''
    
    def __init__(self):
        gtk.TreeStore.__init__(self, gtk.gdk.Pixbuf, gobject.TYPE_PYOBJECT, str, str, gobject.TYPE_PYOBJECT, bool, int, int, bool, str)
        # if its a group: None, parserdata, name, type ('group'), the group object, show pixbuf (False), offline contacts, total contacts, false, ''
        # if its a user:  icon, parserdata, mail, type ('user'), the contact object, show pixbuf (True), 0 if offline 1 otherwise, 0, true if blocked,
        # status
        
class UserListModelList(gtk.ListStore):
    '''this class is the model for a flat userlist'''
    
    def __init__(self):
        gtk.ListStore.__init__(self, gtk.gdk.Pixbuf, gobject.TYPE_PYOBJECT, str, str, gobject.TYPE_PYOBJECT, bool, int, int, bool, str)
        # if its a group: None, parserdata, name, type ('group'), the group object, show pixbuf (False), offline contacts, total contacts, false
        # if its a user:  icon, parserdata, mail, type ('user'), the contact object, show pixbuf (True), 0 if offline 1 otherwise, 0, true if blocked,
        # status
        
class UserList(gtk.TreeView):
    '''the userlist widget'''
    
    # the numbers that identify the columns of the model
    PIX = 0
    LABEL = 1
    ID = 2
    TYPE = 3
    OBJ = 4
    SHOW = 5
    OFFLINE_COUNT = 6   # for group rows
    # notoffline because if i put online you will think only on the NLN status
    NOTOFFLINE_FLAG = 6    # for contact rows
    TOTAL_COUNT = 7
    BLOCKED = 8
    STATUS = 9

    SECS_BETWEEN_FILLS = 3

    __gsignals__ = { 'item-selected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
    (gobject.TYPE_PYOBJECT,)),
    }
    
    def __init__(self, controller, theme, config, isTreeStore = True):
        gtk.TreeView.__init__(self)
        
        self.theme = theme
        self.config = config
        self.controller = controller
        self.unifiedParser = controller.getUnifiedParser()
        self.smileysCache = {}

        # dont you even try to change this value!
        self.isTreeStore = isTreeStore
        
        if isTreeStore:
            self.model = UserListModelTree()
        else:
            self.model = UserListModelList()
        
        self.tooltips = TreeViewTooltips.TreeViewTooltips(self.theme, self, UserList.OBJ, UserList.ID, UserList.TYPE)
            
        self.treeModelFilter = self.model.filter_new()
        self.treeModelFilter.set_visible_func(self.visible_func)
        self.set_model(self.treeModelFilter)
            
        self.filterText = ''
        # a dict with the name of the group as a key and a boolean
        # to True if its expanded and false if collapsed
        self.groupState = {}

        groupColor = self.nickColor
        nickColor = self.nickColor
        statusColor = self.personalMessageColor
        
        self.groupLabel = '<span foreground="' + groupColor + '"><b>%s</b></span>'
        self.userLabel = '<span foreground="' + nickColor + '">%s</span>' +\
        '\n<span size="small" foreground="' + statusColor + '">%s%s</span>'
            
        # label cellrenderer
        self.crt = SmileyRenderer.SmileyRenderer()
        # icon cellrenderer
        self.crp = FancyAvatarRenderer.FancyAvatarRenderer(self.controller)
        
        column = gtk.TreeViewColumn()
        column.set_title('Icons & Text')
        column.set_expand(True)
        
        expColumn = gtk.TreeViewColumn()
        expColumn.set_max_width(16)       
        
        self.append_column(expColumn)
        self.append_column(column)
        self.set_expander_column(expColumn)
        
        column.pack_start(self.crp, False)
        column.pack_start(self.crt)
        
        column.add_attribute(self.crp, 'pixbuf', UserList.PIX)
        column.add_attribute(self.crp, 'blocked', UserList.BLOCKED)
        column.add_attribute(self.crp, 'status', UserList.STATUS)
        column.set_attributes(self.crt, markup=UserList.LABEL)
        column.add_attribute(self.crp, 'visible', UserList.SHOW)
        #self.crt.set_property('ellipsize', pango.ELLIPSIZE_END)
        
        self.set_search_column(UserList.ID)
        self.set_headers_visible(False)
        
        self.connect("row_activated" , self.onRowActivated)
        self.connect("button-press-event" , self.onButtonPress)
        self.connect("row-expanded" , self.onRowExpanded)
        self.connect("row-collapsed" , self.onRowCollapsed)

        self.last_fill = 0
    
    
    # -- CONFIG PROPERTIES -- #
    def getSortNickGroupByContact(self):
        return bool(int(self.config.getUserValue('sortNickGroupByContact','1')))
    sortNickGroup = property(fget=getSortNickGroupByContact)
    
    def getShowByNick(self):
        return bool(int(self.config.getUserValue('showByNick','1')))
    showByNick = property(fget=getShowByNick)

    def getOrderByStatus(self):
        return bool(int(self.config.getUserValue('orderByStatus','1')))
    orderByStatus = property(fget=getOrderByStatus)

    def getShowOffline(self):
        return bool(int(self.config.getUserValue('showOffline','1')))
    showOffline = property(fget=getShowOffline)
    
    def getShowEmptyGroups(self):
        return bool(int(self.config.getUserValue('showEmptyGroups','1')))
    showEmptyGroups = property(fget=getShowEmptyGroups)
    
    def getAvatarsInUserList(self):
        return bool(int(self.config.getUserValue('avatarsInUserList','1')))
    avatarsInUserList = property(fget=getAvatarsInUserList)
        
    def getNickColor(self):
        return self.config.getUserValue('nickColor','#111111')
    nickColor = property(fget=getNickColor)
        
    def getPersonalMessageColor(self):
        return self.config.getUserValue('personalMessageColor','#777777')
    personalMessageColor = property(fget=getPersonalMessageColor)
        
    def getSmallIcons(self):
        return bool(int(self.config.getUserValue('smallIcons','0')))
    smallIcons = property(fget=getSmallIcons)
    
    def getParseSmilies(self):
        return bool(int(self.config.getUserValue('parseSmilies','0')))
    parseSmilies = property(fget=getParseSmilies)

    # -- FILTER STUFF -- #

    def visible_func(self, model, iter):
        obj = model[iter][UserList.OBJ]
        text = self.filterText.lower()
        if text == '' and obj and not self.showOffline and \
           model[iter][UserList.TYPE] == 'user' and obj.status == 'FLN':
            return False
        elif model[iter][UserList.TYPE] == 'group':
            totalCount = model[iter][UserList.TOTAL_COUNT]
            offlineCount = model[iter][UserList.OFFLINE_COUNT]
            if totalCount == offlineCount:
                if (self.showOffline and totalCount > 0) or \
                   self.showEmptyGroups or text != '':
                    return True
                else:
                    return False
            return True
        elif text == '':
            return True
        elif obj and \
             obj.email.find(text) != -1 or \
             obj.nick.find(text) != -1 or \
             obj.alias.find(text) != -1:
            return True

        return False

    def refilter(self):
        self.treeModelFilter.refilter()
        self.expandExpandedGroups()
        
    def setFilterText(self, text):
        self.filterText = text
        self.treeModelFilter.refilter()
        self.expandExpandedGroups()
    
    # -- OTHER FUNCTIONS -- #
    # TODO: group them

    def expandExpandedGroups(self):
        '''iterate over the groups and expand the ones that have True on
        self.groupState'''
        
        if self.isTreeStore:
            for gRow in self.treeModelFilter:
                obj = gRow[UserList.OBJ]
                if not(obj.name in self.groupState and self.groupState[obj.name] == False) and obj.name != 'offline':
                    self.expand_row(gRow.path, False)
        else:
            for row in self.treeModelFilter:
                obj = row[UserList.OBJ]
                if row[UserList.TYPE] == 'group':
                    if not(obj.name in self.groupState and self.groupState[obj.name] == False) and obj.name != 'offline':
                        self.expand_row(row.path, False)
        
    def sortGroupByName(self, groupDict):
        '''this method return a list of groupNames sorted according the 
        configuration'''

        l = groupDict.values() 
        l.sort(lambda x, y: cmp(x.name,y.name)) 
        return l
        
    def sortByStatus(self, groupDict):
        '''receive a groupDict and return a dict of contacts sorted by status'''
        
        online = emesenelib.ContactData.Group(_('Online'))
        offline = emesenelib.ContactData.Group(_('Offline'))
        for group in groupDict.values():
            for user in group.users.values():
                if user.status == 'FLN':
                    offline.setUser(user.email, user)
                else:
                    online.setUser(user.email, user)
                
        return [online, offline]
        
    def getSortedContactMails(self, group):
        '''this method return a list of Contact sorted according the 
        configuration'''
        def sortContact(x , y):
            if x.status == 'FLN':
                return 1
            elif y.status == 'FLN':
                return -1
            else:
                if x.email == y.email:
                    return 0
                elif x.email > y.email:
                    return 1
                else:
                    return -1
       
        contactIds = group.users.values()
        contactIds.sort(sortContact)
        return  [x.email for x in contactIds]
    
    def getSortedContactNick(self, group):
        '''this method return a list of Contact sorted by nick'''
        def sortNick(x , y):
            if x.status == 'FLN':
                return 1
            elif y.status == 'FLN':
                return -1
            else:
                if x.nick == y.nick:
                    return 0
                elif x.nick > y.nick:
                    return 1
                else:
                    return -1
        
        contactIds = group.users.values()
        contactIds.sort(sortNick)
        return [x.email for x in contactIds]

    def getUserSelected(self):
        '''return the user name if a user is selected or an empty string
        if no user is selected'''
        
        (rowType, contact) = self.getSelected()
        
        if rowType == 'user':
            return contact.email
        else:
            return ''
        
    def groupIsSmaller(self, group1, group2):
        '''return True if the group1 < group2 according to the criteria 
        specified by the preferences'''
        
        return group1.name < group2.name
        
    def contactIsSmaller(self, contact1, contact2):
        '''return True if the contact1 < contact2 according to the criteria 
        specified by the preferences'''
        
        return contact1.email < contact2.email
        
    def allContactsOffline(self, contacts):
        '''return True if all contacts are offline'''
        
        return len([x for x in contacts if x.status != 'FLN']) == 0
       
    def add(self, values, iterator=None):
        '''add an item to the model'''
        if self.isTreeStore:
            return self.model.append(iterator, values)
        else:
            return self.model.append(values)
        
    def fill(self, groupDict):
        '''clear and fill the user list, groupDict is a dict containing
        emesenelib.ContactData.Group objects as value and his name as a key'''
        
        if (time.time() - self.last_fill) < UserList.SECS_BETWEEN_FILLS:
            #print 'not filling'
            return

        self.last_fill = time.time()
        self.set_model(None)
        #traceback.print_stack()
        self.model.clear()

        #refresh crp dimention; fill is called when theme prefrences(smallIcons) change, set crp dimention here
        if self.smallIcons:
            self.crp.set_property('dimention', 16)
        else:
            self.crp.set_property('dimention', 32) 

        groupColor = '#000000'
        
        self.groupLabel = '<span foreground="' + groupColor + '"><b>%s</b></span>'
        self.userLabel = '<span foreground="' + self.nickColor + '">%s</span>' +\
        '\n<span size="small" foreground="' + self.personalMessageColor + '">%s%s</span>'
        
        # self.showEmptyGroups ask to config, so we use a temp value
        showEmptyGroups = self.showEmptyGroups
        showOffline = self.showOffline
        
        if self.orderByStatus:
            sortedGroups = self.sortByStatus(groupDict)
        else:
            sortedGroups = self.sortGroupByName(groupDict)
            
        for group in sortedGroups:
            if self.sortNickGroup:
                contacts = self.getSortedContactMails(group)
            else:
                contacts = self.getSortedContactNick(group) 
            groupIter = self.add(self.getGroupRow(group))

            offlineCount = 0
            totalCount = 0

            for contactId in contacts:
                contact = group.users[contactId]
                totalCount += 1
                if contact.status == 'FLN':
                    offlineCount +=1
                # groupIter is not used id we are a list
                self.add(self.getContactRow(contact), groupIter)
                
            self.model.set_value(groupIter, UserList.TOTAL_COUNT, totalCount)
            self.model.set_value(groupIter, UserList.OFFLINE_COUNT, offlineCount)
        
        self.set_model(self.treeModelFilter)
        self.expandExpandedGroups()
                
    def getMenuData(self):
        '''Returns useful data for building menus in the format
        (<selected item type>, [[<item name>], <item's group>])'''
        
        data = self.getSelected()
        
        if data:
            (rowType, obj) = data
            if rowType == 'user':
                return (rowType, obj.email, self.getGroupSelected().name)
            elif rowType == 'group':
                return (rowType, obj.name)
        else:
            return ('',)
          
    def getGroupLabel(self, group):
        '''return the pango string to format the group'''
        return self.groupLabel % group.name
    
    def getGroupRow(self, group):
        '''return the list that contain all the fields to add it to the model'''
        return [None, self.getGroupLabel(group), group.name, 'group', group, False, 0, 0, False, '']
    
    def getContactLabel(self, contact, showAlias=True, tooltip=False):
        '''returns a smileyrenderer list (if tooltip=False) or
        a plain pango string (if tooltip=True)'''
        status = ''
        
        if not self.orderByStatus:
            if contact.status != 'NLN':
                status = '(' + emesenelib.common.reverse_status[contact.status] + ')'

                if contact.personalMessage != '':
                    status += ' - '
        
        if contact.alias != '' and showAlias:
            label = contact.alias    
        elif self.showByNick:
            label = contact.nick
        else:
            label = contact.email
        psm = contact.personalMessage

        hasSmilies = self.parseSmilies
        smallIcons = self.smallIcons

        if smallIcons and not tooltip:
            template = self.userLabel.replace("\n"," - ")
        else:
            template = self.userLabel

        if tooltip:
            # YES! YOU'VE GUESSED! THIS IS A HACK!
            text = self.unifiedParser.getParser( label.replace('\n','') + \
                '\n' + psm.replace('\n','') ).get().split("\n")
            try:
                label = text[0]
                psm = text[1]
            except:
                pass
            return template % (label, status, psm)
        else:
            tuple = (template, label, status, psm)
            
            #send to parser
            parser = self.unifiedParser.getParser(tuple, UserListDataType)
            return parser.get(hasSmilies, self.smileysCache)
          
    def getContactImage(self, contact):    
        '''return the image that will be displayed on the userlist'''
        fallback = 'online'
        if self.avatarsInUserList and \
           self.theme.hasUserDisplayPicture(contact):
            return self.theme.getUserDisplayPicture(contact, 32, 32)
        elif contact.status == 'FLN': 
            if contact.mobile:
                fallback = 'mobile'
            else:
                fallback = 'offline'

        return self.theme.getImage(fallback)
          
    def getContactRow(self, contact):
        '''return the list that contain all the fields to add it to the model'''
        notofflineFlag = 0 # he is offline

        if contact.status != 'FLN':
            notofflineFlag = 1 # he is not offline
        return [self.getContactImage(contact), self.getContactLabel(contact), contact.email, 'user', contact, True, notofflineFlag, 0,
                contact.blocked, contact.status]
          
    def getGroupIter(self, group, add=False):
        '''try to find a group and return the iter, iff not found and add == True
        add it and return the iter, if add==false and not found return None'''
        
        if self.isTreeStore:
            for gRow in self.treeModelFilter:
                obj = gRow[UserList.OBJ]
                if obj.name == group.name:
                    return gRow.iter
        else:
            for row in self.treeModelFilter:
                obj = row[UserList.OBJ]
                if row[UserList.TYPE] == 'group' and obj.name == group.name:
                    return row.iter
        
        if add:
            return self.addGroup(group)            

        return None
          
    def getTupleByIter(self, iterator):
        '''returns a tuple (type, object) corresponding to the given iterator
        type is a string, it can be either 'group' or 'user'
        object is a group or user instance, depending on type'''
        return (self.treeModelFilter[iterator][UserList.TYPE], self.treeModelFilter[iterator][UserList.OBJ])
          
    def getSelected(self):
        '''return a tuple containing the type ("group" or "user") and the object
        or None'''
        
        iterator = self.get_selection().get_selected()[1]
        if iterator == None:
            emesenelib.common.debug('invalid iter')
            return None
            
        return self.getTupleByIter(iterator)
        
    def getGroupSelected(self, iterator=None):
        '''if a group is selected return the group object, if a user is selected,
        return the group in wich its contained'''
        
        if iterator == None:
            iterator = self.get_selection().get_selected()[1]
        objType =  self.treeModelFilter[iterator][UserList.TYPE]
        if objType == 'user':
            return self.treeModelFilter[iterator].parent[UserList.OBJ]
        elif objType == 'group':
            return self.treeModelFilter[iterator][UserList.OBJ]
        
    def updateGroup(self, oldGroup, group):
        '''update the values of group, we use oldGroup because the name could
        have changed, the users inside the group are not modified'''
        # TODO: it erase the children nodes
        for row in self.model:
            obj = row[UserList.OBJ]
            if row[UserList.TYPE] == 'group' and obj.name == oldGroup.name:
                self.model.set_value(row.iter, UserList.LABEL, self.getGroupLabel(group))
                self.model.set_value(row.iter, UserList.OBJ, group)
                break
        
    def updateContact(self, contact):
        '''update the values of a contact, dont use old because the mail cant
        change'''
        doRefilter = False

        if self.isTreeStore:
            for gRow in self.model:
                for row in gRow.iterchildren():
                    obj = row[UserList.OBJ]
                    if row[UserList.TYPE] == 'user' and obj.email == contact.email:
                        row[UserList.PIX] = self.getContactImage(contact)
                        row[UserList.LABEL] = self.getContactLabel(contact)
                        row[UserList.OBJ] = contact
                        row[UserList.BLOCKED] = contact.blocked
                        row[UserList.STATUS] = contact.status

                        # if he is offline and he wasnt offline until now
                        if contact.status == 'FLN' and row[UserList.NOTOFFLINE_FLAG] == 1:
                            # increment the offline count by 1
                            self.model.set_value(gRow.iter, UserList.OFFLINE_COUNT, gRow[UserList.OFFLINE_COUNT] + 1)
                            # set the is offline flas to 0 (he is offline)
                            row[UserList.NOTOFFLINE_FLAG] = 0
                            # if all now, are offline refilter
                            if (gRow[UserList.TOTAL_COUNT] - gRow[UserList.OFFLINE_COUNT]) == 0 or self.orderByStatus:
                                doRefilter = True
                        # if he wasnt offline and the flag says that he was offline
                        elif contact.status != 'FLN' and row[UserList.NOTOFFLINE_FLAG] == 0:
                            # decrease the offline count by 1
                            self.model.set_value(gRow.iter, UserList.OFFLINE_COUNT, gRow[UserList.OFFLINE_COUNT] - 1)
                            # set that he is not offline
                            row[UserList.NOTOFFLINE_FLAG] = 1
                            # if only one contact is online in the group we refresh
                            if (gRow[UserList.TOTAL_COUNT] - gRow[UserList.OFFLINE_COUNT]) == 1 or self.orderByStatus:
                                doRefilter = True
        else:
            for row in self.model:
                obj = row[UserList.OBJ]
                if row[UserList.TYPE] == 'user' and obj.email == contact.email:
                    oldStatus = obj.status
                    row[UserList.PIX] = self.getContactImage(contact)
                    row[UserList.LABEL] = self.getContactLabel(contact)
                    row[UserList.OBJ] = contact
                    row[UserList.BLOCKED] = contact.blocked
                    row[UserList.STATUS] = contact.status

                    # if he is offline and he wasnt offline until now
                    if contact.status == 'FLN' and row[UserList.NOTOFFLINE_FLAG] == 1:
                        # increment the offline count by 1
                        self.model.set_value(row.iter, UserList.OFFLINE_COUNT, row[UserList.OFFLINE_COUNT] + 1)
                        # set the is offline flas to 0 (he is offline)
                        row[UserList.NOTOFFLINE_FLAG] = 0
                        # if all now, are offline refilter
                        if (row[UserList.TOTAL_COUNT] - row[UserList.OFFLINE_COUNT]) == 0 or self.orderByStatus:
                            doRefilter = True
                    # if he wasnt offline and the flag says that he was offline
                    elif contact.status != 'FLN' and row[UserList.NOTOFFLINE_FLAG] == 0:
                        # decrease the offline count by 1
                        self.model.set_value(row.iter, UserList.OFFLINE_COUNT, row[UserList.OFFLINE_COUNT] - 1)
                        # set that he is not offline
                        row[UserList.NOTOFFLINE_FLAG] = 1
                        # if only one contact is online in the group we refresh
                        if (row[UserList.TOTAL_COUNT] - row[UserList.OFFLINE_COUNT]) == 1 or self.orderByStatus:
                            doRefilter = True

        if doRefilter:
            if self.orderByStatus:
                #print '===fill==='
                self.fill(self.controller.msn.contactManager.groups)
            else:
                self.refilter()
    
    def addGroup(self, group):
        '''add a group to the list, return the gtk.TreeIter'''
        
        for row in self.treeModelFilter:
            obj = row[UserList.OBJ]
            if row[UserList.TYPE] == 'group' and self.groupIsSmaller(group, obj):
                if self.isTreeStore:
                    return self.model.insert_before(None, row.iter, self.getGroupRow(group))
                else:
                    return self.model.insert_before(row.iter, self.getGroupRow(group))
        
        if self.isTreeStore:
            return self.model.append(None, self.getGroupRow(group))
        else:
            return self.model.append(self.getGroupRow(group))
    
    def addContact(self, group, contact):
        '''add a contact to the given group, if this is not on the list, its added'''
        
        gRow = self.treeModelFilter[self.getGroupIter(group, True)]
        
        if self.isTreeStore:
            for row in gRow.iterchildren():
                obj = row[UserList.OBJ]
                if row[UserList.TYPE] == 'user' and self.contactIsSmaller(contact, obj):
                    return self.model.insert_before(gRow.iter, row.iter, self.getContactRow(contact))
        else:
            treeModelRow = self.treeModelFilter[gRow.iter]
            treeModelRow = treeModelRow.next
            row = self.treeModelFilter[treeModelRow.iter]
            
            while treeModelRow:
                row = self.treeModelFilter[treeModelRow.iter]
                obj = row[UserList.OBJ]
                # if we reached the next group we add it before
                if row[UserList.TYPE] == 'group':
                    break         
                if row[UserList.TYPE] == 'user' and self.contactIsSmaller(contact, obj):
                    break
                    
                treeModelRow = treeModelRow.next
           
            return self.model.insert_before(row.iter, self.getContactRow(contact))
                    
    # ------------------------ Callbacks -------------------------------------
    
    def onRowActivated(self, treeview, path, view_column):
        self.emit('item-selected', self.getSelected())
        
    def onButtonPress(self, treeview, event):
        if event.button == 3:
            paths = treeview.get_path_at_pos(int(event.x), int(event.y))
            
            if paths == None:
                emesenelib.common.debug('invalid path')
            elif len(paths) > 0:
                iterator = self.treeModelFilter.get_iter(paths[0])
                (rowType,obj) = self.getTupleByIter(iterator)
                
                if rowType == 'user':
                    menu = UserMenu.UserMenu(self.controller , obj, self.getGroupSelected(iterator))
                    menu.popup(None, None, None, event.button, event.time)
                elif rowType == 'group' and not self.orderByStatus:
                    menu = GroupMenu.GroupMenu(self.controller , obj)
                    menu.popup(None, None, None, event.button, event.time)
            else:
                emesenelib.common.debug('empty paths?')
                    
    def onRowExpanded(self,treeview, iterator, path):
        obj = self.treeModelFilter[path][UserList.OBJ]
        self.groupState[obj.name] = True
        
    def onRowCollapsed(self,treeview, iterator, path):
        obj = self.treeModelFilter[path][4]
        self.groupState[obj.name] = False

gobject.type_register(UserList)
        
