# -*- coding: utf-8 -*-

import os
import time
import socket
import string
import urllib
import gobject
import httplib

import Socket
import MsnOIM
import Msnobj
import ContactData
import SoapManager
import XmlTemplates
import Switchboard
import ProfileManager
import SignalHandler

import common

class Msnp( ProfileManager.ProfileManager ):
    '''This class give support to the MSNP13 protocol to use the MSN Messenger network'''

    __gsignals__ = {
        'user-list-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        # when an error happen this signal is emitted with the error name
        # and the description
        'error' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        # when we receive a message, the parameter is the user mail
        'message-received' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
        'connection-problem' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        # disconnected happen when someone login in the same account
        'disconnected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        # disconnect is when a socket error happens
        'connection-closed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        'challenge' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'initial-status-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        # command, tid, params
        'status-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        # command, tid, params
        'status-online' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        # command, tid, params
        'status-offline' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'switchboard-invitation' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'add-notification' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'remove-notification' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'personal-message-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'server-message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'user-disconnected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'logout' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        # signal emited when we change our nick, the parameters are the old and
        # the new nickname
        'self-nick-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'self-status-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'self-personal-message-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'self-current-media-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        
        'new-conversation' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_PYOBJECT)),
        'new-switchboard' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,)),
        'nudge-received' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
            
        'initial-mail-notification' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            ()),
        # From, FromAddr, Subject, MessageURL , PostURL, id
        'new-mail-notification' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'mail-movement-notification' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            ()),
            
        'nick-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        # email, status
        'contact-status-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
         # the bolean value on msnobj-changed indicate if he was offline
         # to not request all the avatars when we come online
        'msnobj-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,gobject.TYPE_BOOLEAN)),
        'user-online' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'user-offline' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'send-message-error' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
            
        'offline-message-waiting' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,)),
        'offline-message-received' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,)),
        
        'display-picture-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'custom-emoticon-transfered' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_PYOBJECT,gobject.TYPE_STRING)),
    }

    def __init__( self, host, port, user, password, \
        userConfigDir, debug = False, proxy = None ):
        '''Contructor, intialize variables.'''
        gobject.GObject.__init__(self)
        
        common.debugFlag = debug

        if proxy == None:
            self.socket = Socket.Socket( host, port )
        else:
            self.socket = Socket.PSocket( host, port, proxy )

        self.user = user.lower()
        self.password = password

        self.cacheDir = userConfigDir+os.sep+'cache'
        
        self.userConfigDir = userConfigDir

        self.proxy = proxy
        self.callbacks = None
        self.nick = ''
 
        self.status = 'FLN'

        self.switchboards = [] # the switchboards to process
        self.switchboardsByTid = {}
        self.demographics = {}

        self.contactManager = ContactData.ContactList( {} )

        self.msnOIM = MsnOIM.MsnOIM( self )

        self.currentMedia = ''
        self.personalMessage = ''
        
        self.msnobj = None # the msnobj for the display picture
        self.picture = None
        
        # p2p features support number
        self.CLIENT_ID = 0x50000000 | 0x4 | 0x2
        
        self.msnObjectsManager = Msnobj.MsnObjectsManager( user )

        self.inboxMessages = 0
        self.inboxUnreadMessages = 0
        self.otherMessages = 0
        self.otherMessagesUnreaded = 0
        
    def _presentation( self ):
        '''this method implement the first stage of the login code because we have 
        to do it two times, return the command, tid and params from the USR command to
        see what we have to do if it is XFR we have to connect to another server if the
        response is USR then whe continue the login'''

        #we connect to the server
        self.socket.connect()

        # send the protocol version we speak
        self.socket.sendCommand( 'VER' , 'MSNP13 CVR0' )
        (command, tid, params) = self.socket.receiveCommand()

        # if we received VER we check if the server wants to speak the same version
        if command == 'VER':
            # if he doesnt want to speak version 13 we raise an exception
            if not params.startswith( 'MSNP13' ):
                raise common.ProtocolError , params
        else:
            # if the server didnt send us VER then something is wrong
            raise common.ProtocolError , command

        # we introduce ourselves
        self.socket.sendCommand( "CVR" , \
            "0x0c0a winnt 5.1 i386 MSNMSGR 8.0.0792 msmsgs " + self.user )
        (command, tid, params) = self.socket.receiveCommand()

        # read the response
        if command != 'CVR':
            raise common.ProtocolError , command

        # introduce the user
        self.socket.sendCommand( "USR" , "TWN I " + self.user )
        return self.socket.receiveCommand()

    def login( self ):
        '''login to the server, after succesfully login you have to change your
            status to appear online'''
        (command, tid, params) = None, None, None
        
        # Redirection
        # We accept as much as 5 redirections
        succeeded = False
        for i in range( 5 ):
            (command, tid, params) = self._presentation()

            if command == 'XFR' and params[ :2 ] == 'NS':
                # the above command tell us where do we have to connect
                #example: XFR 3 NS 207.46.24.39:1863 U D

                self.socket.close()
                try:
                    ns = params.split()[ 1 ].split( ":" ) # 207.46.24.39:1863
                    host = ns[ 0 ]        # 207.46.24.39
                    port = int( ns[ 1 ] ) # int( 1863 )
                except:
                    raise common.RedirectionError, 'Incorrect parameters: ' + params

                if self.proxy == None:
                    self.socket = Socket.Socket( host, port )
                else:
                    self.socket = Socket.PSocket( host, port, self.proxy )

                # and do all over again
            else:
                succeeded = True
                break
            
        if not succeeded:
            raise common.RedirectionError, 'Too many redirections'
        
        # Recieve GCF payload if any
        if command == 'GCF':
            self.socket.receivePayload( int( params ) )

        (command, tid, params) = self.socket.receiveCommand()

        # if we didnt get a USR response..
        if command != 'USR':
            raise common.AuthError, str( command ) + ' received. Expecting USR - TWN'

        # we get the hash
        try:
            hash = params.split( "TWN S " )[ 1 ]
        except Exception, e:
            print e
            raise common.AuthError, 'Incorrect USR response parameters: ' + params
        # start the passport authentication
        self.passportid = self.passportAuth( hash )
        try:
            self.t = self.passportid.split( '&p=')[ 0 ]
            self.t = self.t[2:]
            self.MSPProf = self.passportid.split( '&p=')[ 1 ]
        except:
            raise common.AuthError, 'Incorrect passportid: ' + self.passportid

        # we introduce ourselves again
        self.socket.sendCommand( "USR" , "TWN S " + self.passportid )
        (command, tid, params) = self.socket.receiveCommand()
        # if something is wrong raise an exception
        try:
            if command != "USR" and params[ 0:2 ] != "OK":
                raise common.AuthError
        except:
            raise common.AuthError, '"%s - %s" received. Expecting USR - OK' % (command, params)

        data = self.socket.receive()        
        if self.proxy == None:
            while data != '':
                data = self.socket.receive()
                if data.startswith( "MSPAuth" ):
                    try:
                        self.MSPAuth = data.split( "MSPAuth: " )[ 1 ]
                    except:
                        raise common.AuthError, 'Incorrect MSPAuth data: ' + data
                else:
                    try:
                        key, value =  data.split( ":" )
                        self.demographics.update( {key.strip():value.strip()} )
                    except:
                        pass
        else:
            # if we are using the HTTP Method all the data comes
            # in one package (nice :)
            try:
                payload = self.socket.receive()
                self.MSPAuth = payload.split( "MSPAuth: " ) \
                    [ 1 ].split( '\r\n' )[ 0 ]
            except:
                raise common.AuthError, 'Incorrect MSPAuth data: ' + data
        self.socket.onLogin = False

        # ------ logged in --------
        common.debug("logged in", 'core')
        self.soapManager = SoapManager.SoapManager( self.MSPAuth, self.MSPProf )
        self.soapManager.start()
        # Get the lists
        mlCache = self.getCacheFileData( self.user + "_ml.xml" )
        diCache = self.getCacheFileData( self.user + "_di.xml" )
        
        if mlCache and diCache:
            common.debug("parsing membership list", 'core')
            start = time.time()
            self.contactManager.setMembershipListXml( mlCache )
            common.debug("done:" + str(time.time() - start), 'core')

            common.debug("parsing dynamic items", 'core')
            start = time.time()
            self.contactManager.setDynamicItemsXml( diCache )
            self.getNickFromDynamicItems( diCache )
            common.debug("done:" + str(time.time() - start), 'core')

            self.emit( 'user-list-change' )
        else:
            #if not self.getMembershipListSync():
            #    raise common.LoginError, 'Couldn\'t retrieve membership list'
            #if not self.getDynamicItemsSync():
            #    raise common.LoginError, 'Couldn\'t retrieve dynamic items'
            self.getMembershipListSync()
            self.getDynamicItemsSync()

        # send privacy status
        self.socket.sendCommand( "BLP" , "BL" )
        
        # send our contacts (as of rev 411 only FL contacts are sent)
        for i in self.createADL():
            self.socket.sendPayloadCommand( "ADL", '', i )
        
        # change the personal message
        self.changePersonalMessage('')
        
        # change the nickname
        self.changeNick( self.nick, sendSOAP=False )

        # connect some callbacks
        self.callbacks = SignalHandler.SignalHandler( self )
        self.connect( 'challenge' , self.callbacks.challenge )
        self.connect( 'status-change' , self.callbacks.statusChange )
        self.connect( 'initial-status-change' , self.callbacks.statusChange )
        self.connect( 'status-online' , self.callbacks.statusOnline )
        self.connect( 'status-offline' , self.callbacks.statusOffline )
        self.connect( 'switchboard-invitation' , self.callbacks.switchboardInvitation )
        self.connect( 'server-message' , self.callbacks.serverMessage )
        
        if mlCache and diCache:
            self.getMembershipListAsync()
            self.getDynamicItemsAsync()

    def passportAuth( self , hash ):
        '''do the passport authenticaton, this is done connecting to loginnet.passport.com:443
        and sending a XML message described on XmlTemplates.passport'''

        common.debug('PASSPORT begin', 'core')

        # unquote the hash
        ticket = urllib.unquote( hash ).replace( "," , "&amp;" )
        # replace the %s in the string
        body = XmlTemplates.passport%( self.user, self.password, ticket )

        #create the headers
        headers = { \
        "Accept" :  "text/*" , \
        "User-Agent" : "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)" , \
        "Host" : "loginnet.passport.com" , \
        "Content-Length" : str( len( body ) ) , \
        "Connection" : "Keep-Alive" , \
        "Cache-Control" : "no-cache" , \
        }

        _server="loginnet.passport.com"
        _url="/RST.srf"
        
        succeeded = False
        for i in range( 5 ):
            # send the SOAP request
            for i in range( 3 ):
                try:
                    conn = httplib.HTTPSConnection(_server,443)
                    conn.request("POST", _url, body, headers)
                    response = conn.getresponse()
                    common.debug( str( response.status ) + " " + response.reason, 'core' )
                    break
                except Exception, e:
                    #print str( e )
                    #raise e
                    pass

            data = response.read()

            if data.find( '<faultcode>psf:Redirect</faultcode>' ) > 0:
                from urlparse import urlparse
                _url=urlparse( data.split( '<psf:redirectUrl>' )[ 1 ].split( '</psf:redirectUrl>' )[ 0 ] )
                # fixed according to http://docs.python.org/lib/module-urlparse.html
                _server=_url[ 1 ]
                _url=_url[ 2 ]
                common.debug( 'Redirect to: %s %s' % ( _server, _url ), 'core' )
            else:
                succeeded = True
                break
        
        if not succeeded:
            raise common.AuthError, 'Too many redirections'

        # try to get the ticket from the received data
        try:
            twnticket = data.split( "<wsse:BinarySecurityToken Id=\"PPToken1\">" )\
                            [ 1 ].split( "</wsse:BinarySecurityToken>" )[ 0 ]
            twnticket = twnticket.replace( "&amp;" , "&" )
        except Exception, e:
            common.debug( e, 'core' )
            common.debug( data, 'core' )
            # try to get the faultstring
            try:
                faultstring = data.split( "<faultstring>" )\
                    [ 1 ].split( "</faultstring>" )[ 0 ]
            except:
                faultstring = ''
            raise common.AuthError, faultstring

        return twnticket

    def ping( self ):
        '''send a ping to the server and register the timestamp of the command'''
        try:
            self.socket.ping()
        except (IOError, socket.error):
            self.emit( 'connection-closed' )
            
    def checkConnection( self ):
        '''return true if the time between the last ping and the response is
        lower than MAX_PING_DELAY'''
        
        if (self.socket.lastqng < self.socket.lastpng) and \
           (int(time.time()) - self.socket.lastpng) > self.socket.MAX_PING_DELAY:
            return False
        
        return True

    def process( self ):
        '''read a command from the server and process it'''

        SoapManager.process()
        
        # call the process method of every switchboard
        for sb in self.switchboards:
            if sb.status != 'closed':
                sb.process()
        
        #check connection
        if self.socket.lastpng == 0:
            self.ping()
        elif not self.checkConnection():
            self.emit( 'connection-problem' )
            print 'connection problem'
        
        if (int( time.time() ) - self.socket.lastpng) > self.socket.SECS_BETWEEN_PINGS:
            self.ping()
            
        fds = self.socket.select()

        if len( fds[ 0 ] ) == 1:
            try:
                (command, tid, params) = self.socket.receiveCommand()
            except IOError:
                self.emit( 'connection-closed' )
        else:
            return
        
        if len( fds[ 1 ] ) == 1:
            if command == 'CHL':
                self.emit( 'challenge', command, tid, params )

        if command == 'ILN':
            self.emit( 'initial-status-change', command, tid, params )
        elif command == 'NLN':
            self.emit( 'status-online', command, tid, params )
        elif command == 'FLN':
            self.emit( 'status-offline', command, tid, params )
        elif command == 'RNG':
            self.emit( 'switchboard-invitation', command, tid, params )
        elif command == 'ADL':
            if params != 'OK':
                payload = self.socket.receivePayload( int( params ) )
                self.emit( 'add-notification', command, tid, params, payload )
        elif command == 'RML':
            if params != 'OK':
                payload = self.socket.receivePayload( int( params ) )
                self.emit( 'remove-notification', command, tid, params, payload )
        elif command == 'UBX':
            payload = self.socket.receivePayload( int( params ) )
            common.debug( '<<< ' + payload, 'core' )
            if int( params ) > 0:
                try:
                    pm = payload.split( '<PSM>' )[ 1 ].split( '</PSM>' )[ 0 ]
                    pm = common.unescape( pm )
                    self.contactManager.setContactPersonalMessage( tid, pm )
                    
                    media = payload.split( '<CurrentMedia>' )[ 1 ].split( '</CurrentMedia>' )[ 0 ]

                    mhead = media.find('\\0Music\\01\\0')
                    if mhead != -1:
                        media = '♫ ' + media[mhead+12:]
                        margs = media.split('\\0')
                        media = margs[0]
                        for args in range(1, len(margs)):
                            media = media.replace( '{%s}' %str(args-1), margs[args])
                        media = common.unescape(media)
                        self.contactManager.setContactPersonalMessage( tid, media )
                    else:
                        self.contactManager.setContactPersonalMessage( tid, pm )

                    self.emit('personal-message-changed', tid, pm)
                    contact = self.contactManager.getContact( tid )
                    self.emit( 'user-attr-changed', contact )
                except Exception, e:
                    #raise common.ProtocolError, 'Unable to handle UBX: ' + str( params )
                    print 'Unable to handle UBX: ' + str( params )
        elif command == 'XFR':
            #XFR 32 SB 65.54.171.31:1863 CKI 1743299383.52212212.219110167 U messenger.msn.com\r\n
            self.switchboardsByTid[int(tid)].setConnectionString(string.join([command, tid, params]))
        elif command == 'MSG':
            try:
                payload = self.socket.receivePayload( int( params.split()[ 1 ] ) )
                self.emit( 'server-message', command, tid, params, payload )

            except Exception, e:
                common.debug( 'Exception in msnp.process, continuing', 'core' )
                common.debug( '(EE) ' + str( e ), 'core' )
        elif command == 'OUT':
            self.emit( 'user-disconnected', tid, params )
        elif command == 'QNG':
           self.socket.onQng(tid)
            
    def changeStatus( self , status ):
        '''change the current status'''
        
        if status == 'FLN' or status == 'offline':
            return
        
        msnobj = ''
        
        if self.msnobj:
            msnobj = ' ' + self.msnobj.quote()
        else:
            msnobj = ' 0'
        #print msnobj
        clientid = ' ' + str(self.CLIENT_ID)

        if common.status_table.has_key( status ):
            self.socket.sendCommand( "CHG" , common.status_table[ status ] + clientid + msnobj )
            self.status = common.status_table[ status ]
            self.emit( 'self-status-changed', self.status )
        elif common.reverse_status.has_key( status ):
            self.socket.sendCommand( "CHG" , status + clientid + msnobj )
            self.status = status
            self.emit( 'self-status-changed', self.status )

    def sendDL( self,command, email, type ):
        '''send ADL or RML
        #values of l
        #1 = Forward List
        #2 = Allow List
        #4 = Block List
        #8 = Reverse List (not to be tampered with)
        #16 = Pending list

        FL Forward List Users who were added to your contact list
        RL Reverse List Users who added you to their contact list
        AL Allow List Users who are able to see your status
        BL Block List Users who are blocked from seeing your status'''

        try:
            (user, domain) = email.split( '@' )
        except:
            return

        payload = '<ml><d n="' + domain + '"><c n="' + user + '" t="1" l="' + str( type ) + '" /></d></ml>'

        self.socket.sendPayloadCommand( command , '' , payload  )

    def logout( self ):
        '''disconnect from the server'''
        self.emit( 'logout' )
        try:
            self.socket.sendText( 'OUT\r\n' )
        except:
            pass

        self.contactManager = ContactData.ContactList( {} )
        # Send the soapmanager a stop signal
        SoapManager.put( 'quit' )
        SoapManager.process()
        
    def createADL( self ):
        '''Create a XML String with all the contacts we have for the ADL Command'''

        return self.contactManager.getADL()
        
    def newSwitchboard( self ):
        '''create a new Switchboard and return it'''
        tid = self.socket.getTid()
        new = Switchboard.Switchboard(tid, self, 'requested')
        self.on_new_switchboard(new)
        self.emit('new-switchboard', new)
        self.switchboardsByTid[tid] = new
        self.socket.sendCommand('XFR', 'SB')

        return new

    def removeClosedSwitchboards(self):
        closed = []
        for sb in self.switchboards:
            if sb.status == 'closed':
                closed.append(sb)
        for sb in closed:
            self.switchboards.remove(sb)
        

    def on_display_picture_received(self, switchboard, msnobj, data, email):
        '''called when a display picture has been received'''
        contact = self.contactManager.getContact( email )
        filename = contact.displayPicturePath
        open(os.path.join(self.cacheDir, filename), 'wb').write(data)
        print "Updated avatar path - ", filename
        print "emitting display-picture-changed"
        self.emit('display-picture-changed', email)

    def on_custom_emoticon_received(self, switchboard, msnobj, data, email):
        '''called when a custom emoticon has been received'''
        open(msnobj.filename,'wb').write(data)
        self.emit('custom-emoticon-transfered', email, msnobj, msnobj.filename)

    def on_new_switchboard(self, switchboard):
        '''called when a new switchboad is created, its not a callback from a signal'''
        self.switchboards.append(switchboard)
        switchboard.connect('display-picture-received', self.on_display_picture_received)
        switchboard.connect('custom-emoticon-data-received', self.on_custom_emoticon_received)
    
    def getSwitchboard( self, email ):
        '''try to retrieve an existing switchboard that has email as the first
        user and its not a group chat. Clean up closed switchboards and create
        a new switchboard if no existing switchboard matches.'''

        self.removeClosedSwitchboards()
        switchboard = None
        for sb in self.switchboards:
            if not sb.isGroupChat() and sb.firstUser == email:
                switchboard = sb

        if not switchboard:
            switchboard = self.newSwitchboard()
            switchboard.invite(email)
        return switchboard

    def getGroups( self ):
        '''return the groups in the userlist'''

        return self.contactManager.groups
    
    def getGroupNames(self):
        '''Returns a list with the group names in the user list'''
        
        return self.contactManager.getGroupNames()

    def checkPending( self ):
        '''Return all the pending users'''

        l = []
        lists = self.contactManager.lists
        for email in lists[ 'Pending' ]:
            if email not in lists[ 'Block' ] and email not in lists[ 'Allow' ]:
                l.append( email )

        return l

    def cacheFileExist( self, fileName ):
        '''return true if the file exist in the cache directory'''

        return os.path.isfile( self.cacheDir + os.sep + fileName )

    def newCacheFile( self, fileName, data ):
        '''create the file and put data on it'''

        f = open( self.cacheDir + os.sep + fileName, 'w' )
        f.write( data )
        f.close()

    def getCacheFileData( self, fileName ):
        '''get the data in filename, return None otherwise'''

        if self.cacheFileExist( fileName ):
            f = open( self.cacheDir + os.sep + fileName, 'r' )
            data = f.read()
            f.close()
            return data
        else:
            return None

    def setDisplayPicture(self, filename):
        '''try to open the picture and set the msnobj'''
        if filename == '':
            self.msnobj = None
        else:
            self.msnobj = self.msnObjectsManager.create( '', filename, type=Msnobj.Msnobj.DISPLAY_PICTURE )
            
        if self.status != 'FLN':
            self.changeStatus( self.status )
            
    def createCustomEmoticon( self, shortcut, filename ):
        self.msnObjectsManager.create( shortcut, filename, type=Msnobj.Msnobj.CUSTOM_EMOTICON )
        
    def getMsnObjectsManager( self ):
        return self.msnObjectsManager
    
    def getUser( self ):
        return self.user
    
    def getNick( self ):
        return self.nick

gobject.type_register(Msnp)

