import os
import gettext
import datetime

_ = gettext.gettext

class Log( object ):
    '''a class that contain LogLine items, allow filters and stuff like that'''

    reYear = ''
    reHour = ''
    reEvent = ''
    reMail = ''

    def __init__( self ):
        self.logs = []
        # a list of the mails in the logs
        self.mails = []

    def parseFile( self, fileName ):
        '''open a file and add the lines to the log list'''

        try:
            fd = file( fileName, 'r' )

            for line in fd.readlines():
                if self.validateLine( line ):
                    self.logs.append( self.parseLine( line[:-len(os.sep)] ) )

            return True
        except IOError:
            return False

    def parseLine( self, line ):
        '''parse a line and return a LogLine object'''
        # call to self.validateLine before this!
        # we dont want to return None or something like
        # that, also we dont want exception all over the place :)

        items = line.split( ' ', 5 )

        if len( items ) == 4:
            (year, hour, event, email, extra) = items + [ '' ]
        elif len( items ) == 5:
            (year, hour, event, email, extra) = items 
            
        if email not in self.mails:
            self.mails.append( email )

        return LogLine( year, hour, event, email, extra )

    def validateLine( self, line ):
        '''return True if the line has the first tree correct fields,
        these are:
        07-07-11 20:37:27 event-nam user@hotmail.com'''

        # we return True for now
        # TODO: add this

        return True

class LogLine( object ):
    '''an object containig the data of a single log line'''

    # the event names where the extra field is quoted
    quotedExtras = []
    quotedExtras.append( 'self-nick-changed' )
    quotedExtras.append( 'personal-message-changed' )
    quotedExtras.append( 'self-personal-message-changed' )
    quotedExtras.append( 'self-current-media-changed' )
    quotedExtras.append( 'nick-changed' )

    # the event names where the extra field is a status
    statusExtras = []
    statusExtras.append( 'status-change' )
    statusExtras.append( 'self-status-changed' )
    statusExtras.append( 'contact-status-change' )
    statusExtras.append( 'user-online' )
    statusExtras.append( 'user-offline' )

    # a dict that maps the status to more readable ones
    statusDict = {}
    statusDict[ 'NLN' ] = _( 'Online' )
    statusDict[ 'AWY' ] = _( 'Away' )
    statusDict[ 'BSY' ] = _( 'Busy' )
    statusDict[ 'BRB' ] = _( 'Be right back' )
    statusDict[ 'PHN' ] = _( 'On the phone' )
    statusDict[ 'LUN' ] = _( 'Lunch' )
    statusDict[ 'HDN' ] = _( 'Invisible' )
    statusDict[ 'IDL' ] = _( 'Idle' )
    statusDict[ 'FLN' ] = _( 'Offline' )

    def __init__( self, year, hour, event, email, extra ):
        (year, month, day, hour, minute, second) = year.split( '-' ) + hour.split( ':' )
        self.date = datetime.datetime( int(year)+2000, int(month), int(day), int(hour), int(minute), int(second) )
        self.event = event
        self.email = email
        self.extra = extra

    def __str__( self ):
        return '<LogLine ' + str(self.date) + ' ' + self.event + ' ' + self.email + '>'

    def formatExtra( self ):
        '''return the extra field formated to user view'''

        if self.event in LogLine.quotedExtras:
            return self.extra.replace( '%20', ' ' )
        elif self.event in LogLine.statusExtras and self.extra in LogLine.statusDict:
            return LogLine.statusDict[ self.extra ]
        elif self.extra == '':
            return self.extra

        print '? ' + self.event 

        return self.extra

if __name__ == '__main__':
    l = Log()
    l.parseFile( '11-07-07.log' )
    for logLine in l.logs:
        print logLine
