import os
import sys
import gtk
import gettext

import Log
import LogView
import FilterPanel
import LogViewerMenu

gettext.install('logViewer',os.curdir + os.sep + 'po',unicode=False)

class LogViewerWindow( gtk.Window ):
    '''the main window of the log viewer'''

    def __init__( self ):
        gtk.Window.__init__( self )

        self.set_title( _( 'Emesene log viewer' ) )
        self.set_default_size( 600,400 )
        self.set_border_width( 5 )

        # initialization
        self.vbox = gtk.VBox()
        self.menu = LogViewerMenu.LogViewerMenu()
        self.paned = gtk.HPaned()

        self.filters = FilterPanel.FilterPanel( LogView.LogView.eventNames, [] )
        self.filters.mailFilters.connect( 'mail-filter-changed', self.mailFilterChanged )
        self.filters.eventFilters.connect( 'event-filter-changed', self.eventFilterChanged )

        self.menu.connect( 'open-file', self.onOpenFile )
        self.menu.connect( 'clear', self.onClear )
        self.menu.connect( 'quit', self.onQuit )

        self.scrollView = gtk.ScrolledWindow()
        self.logView = LogView.LogView()

        self.mails = []
        # set attributes

        self.paned.set_border_width( 5 )

        # add all the widget inside eachothers

        self.scrollView.add( self.logView )

        self.paned.add1( self.filters )
        self.paned.add2( self.scrollView )

        self.vbox.pack_start( self.menu, False, False )
        self.vbox.pack_start( self.paned )
        self.vbox.pack_start( gtk.Statusbar(), False, False )
        self.add( self.vbox ) 
        
        self.vbox.show_all()

    def showFile( self, fileName ):
        '''parse a file and show the content'''

        log = Log.Log()
        
        if log.parseFile( fileName ):
            for logLine in log.logs:
                self.logView.add( logLine )

            for mail in log.mails:
                if mail not in self.mails:
                    self.filters.mailFilters.addMail( mail )
                    self.mails.append( mail )
        else:
            # TODO: show this on a message dialog
            print _( 'Can\'t open the file' )
    
    def mailFilterChanged( self, mailFilter, mail, status ):
        if status == True:
            self.logView.addVisibleMail( mail )
        else:   
            self.logView.delVisibleMail( mail )

    def eventFilterChanged( self, eventFilter, event, status ):
        if status == True:
            self.logView.addVisibleEvent( event )
        else:   
            self.logView.delVisibleEvent( event )

    def onOpenFile( self, widget ):
        dialog = gtk.FileChooserDialog( _( 'Open a log file' ), self, gtk.FILE_CHOOSER_ACTION_OPEN, ( _( 'Open' ), gtk.RESPONSE_OK, _( 'Cancel' ),
        gtk.RESPONSE_CANCEL) )
        dialog.set_select_multiple( True )

        filter = gtk.FileFilter()
        filter.add_pattern( '*.log' )
        filter.set_name( '*.log' )
        dialog.add_filter( filter )

        response = dialog.run()
        dialog.hide()

        if response == gtk.RESPONSE_OK:
            for file in dialog.get_filenames():
                self.showFile( file )
    
    def onClear( self, widget ):
        self.logView.model.clear()
        self.filters.clearMails( [] )
        self.mails = []
        self.logView.visibleMails = []

    def onQuit( self, widget ):
        sys.exit( 0 )

if __name__ == '__main__':
    w = LogViewerWindow()
    w.show()
    gtk.main()
