# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import time
import string

import Plugin
class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        Plugin.Plugin.__init__( self, controller, msn )
        
        self.description = 'Save all avatars'
        self.authors = { 'Mariano Guerra' : 'luismarianoguerra  at gmail dot com' }
        self.website = 'http://emesene.org'
        self.displayName =  'AvatarHistory'
        self.name = 'AvatarHistory'
        
        self.config = controller.getConfig()
        self.config.readPluginConfig(self.name)
        self.path = self.config.getPluginValue( self.name, 'path', self.msn.cacheDir )
        self.fileName = time.strftime( "%d-%m-%y" ) + ".log"
        self.ids = []
        
        self.signals = {}
        self.signals[ 'new-switchboard' ] = self.on_new_switchboard
    
    def start( self ):
        '''start the plugin'''

        self.enabled = True
        
        for (key,value) in self.signals.iteritems():
            self.ids.append( self.msn.connect( key, value ) )
        
    def stop( self ):    
        '''stop the plugin'''
        
        for id in self.ids:
            self.msn.disconnect(id)
        
        self.enabled = False
        
    def action( self ):
        pass        
    
    def check( self ):
        '''check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )'''
        
        return ( True, 'Ok' )

    def getTimeStamp( self ):
        return time.strftime( "%y%m%d %H%M%S" )
            
    def on_new_switchboard(self, msn, switchboard):
        '''called when a new switchboard is created'''
        switchboard.connect('display-picture-received', self.on_display_picture_received)

    def on_display_picture_received(self, switchboard, msnobj, data, email):
        mail = email.replace('@','_').replace('.', '_')
        file_ = file(self.path + os.sep + mail + ' ' + self.getTimeStamp(),'wb')
        file_.write(data)
        file_.close()

